#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file PSub3.py
 @brief EV3 Color Sensor
 @date $Date$


"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist


# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
psub3_spec = ["implementation_id", "PSub3", 
		 "type_name",         "PSub3", 
		 "description",       "EV3 Color Sensor", 
		 "version",           "1.0.0", 
		 "vendor",            "Aizu", 
		 "category",          "Category", 
		 "activity_type",     "STATIC", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 "conf.default.FowardSpeed", "0.04",
		 "conf.default.BackSpeed", "0.05",
		 "conf.default.BackTime", "1500",
		 "conf.default.TurnSpeed", "0.78",
		 "conf.default.TurnTime", "2000",

		 "conf.__widget__.FowardSpeed", "slider.0.01",
		 "conf.__widget__.BackSpeed", "slider.0.01",
		 "conf.__widget__.BackTime", "slider.1",
		 "conf.__widget__.TurnSpeed", "slider.0.01",
		 "conf.__widget__.TurnTime", "slider.1",
		 "conf.__constraints__.FowardSpeed", "0<=x<=0.5",
		 "conf.__constraints__.BackSpeed", "0<=x<=0.5",
		 "conf.__constraints__.BackTime", "0<=x<=10000",
		 "conf.__constraints__.TurnSpeed", "-8.5<=x<=8.5",
		 "conf.__constraints__.TurnTime", "0<=x<=10000",

         "conf.__type__.FowardSpeed", "float",
         "conf.__type__.BackSpeed", "float",
         "conf.__type__.BackTime", "int",
         "conf.__type__.TurnSpeed", "float",
         "conf.__type__.TurnTime", "int",

		 ""]
# </rtc-template>

##
# @class PSub3
# @brief EV3 Color Sensor
# 
# 
class PSub3(OpenRTM_aist.DataFlowComponentBase):
	
	##
	# @brief constructor
	# @param manager Maneger Object
	# 
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

		#color_arg = [None] * ((len(RTC._d_TimedString) - 4) / 2)
		#self._d_color = RTC.TimedString(*color_arg)
		self._d_color = RTC.TimedString(RTC.Time(0,0), "")
		"""
		"""
		self._colorIn = OpenRTM_aist.InPort("color", self._d_color)
		#vel_arg = [None] * ((len(RTC._d_TimedVelocity2D) - 4) / 2)
		#self._d_vel = RTC.TimedVelocity2D(*vel_arg)
		self._d_vel = RTC.TimedVelocity2D(RTC.Time(0,0),RTC.Velocity2D(0.0, 0.0, 0.0))
		"""
		"""
		self._velOut = OpenRTM_aist.OutPort("vel", self._d_vel)


		


		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		"""
		
		 - Name:  FowardSpeed
		 - DefaultValue: 0.04
		"""
		self._FowardSpeed = [0.04]
		"""
		
		 - Name:  BackSpeed
		 - DefaultValue: 0.05
		"""
		self._BackSpeed = [0.05]
		"""
		
		 - Name:  BackTime
		 - DefaultValue: 1500
		"""
		self._BackTime = [1500]
		"""
		
		 - Name:  TurnSpeed
		 - DefaultValue: 0.78
		"""
		self._TurnSpeed = [0.78]
		"""
		
		 - Name:  TurnTime
		 - DefaultValue: 2000
		"""
		self._TurnTime = [2000]
		
		# </rtc-template>


		 
	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry() 
	# 
	# @return RTC::ReturnCode_t
	# 
	#
	def onInitialize(self):
		# Bind variables and configuration variable
		self.bindParameter("FowardSpeed", self._FowardSpeed, "0.04")
		self.bindParameter("BackSpeed", self._BackSpeed, "0.05")
		self.bindParameter("BackTime", self._BackTime, "1500")
		self.bindParameter("TurnSpeed", self._TurnSpeed, "0.78")
		self.bindParameter("TurnTime", self._TurnTime, "2000")
		
		# Set InPort buffers
		self.addInPort("color",self._colorIn)
		
		# Set OutPort buffers
		self.addOutPort("vel",self._velOut)
		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		
		return RTC.RTC_OK
	
	#	##
	#	# 
	#	# The finalize action (on ALIVE->END transition)
	#	# formaer rtc_exiting_entry()
	#	# 
	#	# @return RTC::ReturnCode_t
	#
	#	# 
	#def onFinalize(self):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The startup action when ExecutionContext startup
	#	# former rtc_starting_entry()
	#	# 
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The shutdown action when ExecutionContext stop
	#	# former rtc_stopping_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The activated action (Active state entry action)
	#	# former rtc_active_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	# 
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onActivated(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The deactivated action (Active state exit action)
	#	# former rtc_active_exit()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onDeactivated(self, ec_id):
	#
	#	return RTC.RTC_OK
	
		##
		#
		# The execution action that is invoked periodically
		# former rtc_active_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onExecute(self, ec_id):
	
		# J[ZT[̒lXVĂꍇ
		if self._colorIn.isNew():
			# J[ZT[̒l擾
			self._d_color = self._colorIn.read()

			# J[ZT[̒l2܂3̏ꍇ
			if self._d_color.data == "2" or self._d_color.data == "3":
				# ~w
				self._d_vel.data = RTC.Velocity2D(0.0, 0.0, 0.0)
				self._velOut.write()
				# Sleep
				time.sleep(0.5);

				# ގw
				self._d_vel.data = RTC.Velocity2D(float(self._BackSpeed[0]) * -1, 0.0, 0.0)
				self._velOut.write()
				# Sleep
				time.sleep(int(self._BackTime[0]) / 1000);
				
				# J[ZT[̒l2̏ꍇ
				if self._d_color.data == "2":
					# w
					self._d_vel.data = RTC.Velocity2D(0.0, 0.0, float(self._TurnSpeed[0]))
					self._velOut.write()
				else:
					# Ew
					self._d_vel.data = RTC.Velocity2D(0.0, 0.0, float(self._TurnSpeed[0]) * -1)
					self._velOut.write()

				# Sleep
				time.sleep(int(self._TurnTime[0]) / 1000);
		
		# Oiw
		self._d_vel.data = RTC.Velocity2D(float(self._FowardSpeed[0]), 0.0, 0.0)
		self._velOut.write()

		return RTC.RTC_OK
	
	#	##
	#	#
	#	# The aborting action when main logic error occurred.
	#	# former rtc_aborting_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The error action in ERROR state
	#	# former rtc_error_do()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onError(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The reset action that is invoked resetting
	#	# This is same but different the former rtc_init_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The state update action that is invoked after onExecute() action
	#	# no corresponding operation exists in OpenRTm-aist-0.2.0
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#

	#	#
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The action that is invoked when execution context's rate is changed
	#	# no corresponding operation exists in OpenRTm-aist-0.2.0
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK
	



def PSub3Init(manager):
    profile = OpenRTM_aist.Properties(defaults_str=psub3_spec)
    manager.registerFactory(profile,
                            PSub3,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    PSub3Init(manager)

    # Create a component
    comp = manager.createComponent("PSub3")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

